Import MaxGUI.MaxGUI
Rem
		ButtonExtra
End Rem

Rem
bbdoc:Create a Special button/field
about:Create a combination of label+button/textfield/slider/progress bar/combobox

Style defines what type of 'extra' gadget you want

[
* GADGET_TEXTFIELD | Creates a textfield
* GADGET_BUTTON | Creates a basic button
* GADGET_SLIDER | Creates a slider
* GADGET_COMBOBOX | Creates a combobox
* GADGET_PROGBAR | Creates a progress bar
]
You can use standard MaxGUI commands: SetSliderRange, AddGadgetItems and so on.<br>

You can 'send' a message to a single extra-gadget using this pseudo-commands:
[
* <#TFIELD#> | Allow to change the content of a textfield
* <#SIZE#> | Allow to resize the button/textfield gadget. The value is expressed in %%
]

<#VALUE#> is a placeholder for slider value<br>
When you move the slider, the text will change automatically.

End Rem
Function CreateButtonExtra:tbuttonExtra(msg:String,def:String,x:Int,y:Int,w:Int,h:Int,group:tgadget,style:Int=GADGET_TEXTFIELD)
Return tButtonExtra.Create(msg,def,x,y,w,h,group,style)
End Function

Type tButtonExtra Extends Tproxygadget


	Field panel:tgadget
	Field label:tgadget
	Field button:tgadget
	Field style:Int,Text$
	

	Method CleanUp()
		RemoveHook EmitEventHook,EventHandler,Self
		If label FreeGadget label;label = Null
		If button FreeGadget button;button = Null
		If panel FreeGadget panel;panel = Null
		Super.CleanUp()
	End Method
	
	Method ItemExtra:Object(index:Int=0)
		If style=GADGET_COMBOBOX
		Return GadgetItemExtra(button,index)
		End If
	End Method

	
	Method SetItem(index:Int,Text:String,tip:String,icon:Int,extra:Object,flags:Int)
		If style=GADGET_COMBOBOX
		ModifyGadgetItem button,index,Text,flags,icon,tip,extra
		End If
	End Method

	
	Method RemoveItem:Int(index:Int)
		If style=GADGET_COMBOBOX
					RemoveGadgetItem button,index
			End If
	End Method
	
	Method ItemCount:Int()
		If style=GADGET_COMBOBOX
			Return CountGadgetItems(button)
		End If
	End Method

	
	Method SelectItem:Int(it:Int,st:Int)
			If style=GADGET_COMBOBOX
					SelectGadgetItem button,it
			End If
	End Method
	
	Method InsertItem(index:Int,Text:String="",tip:String="",icon:Int=-1,extra:Object=Null,flags:Int=0)
		If style=GADGET_COMBOBOX
					AddGadgetItem button,Text,flags,icon,tip,extra
		End If
	End Method

	Method SetRange:Int(a:Int,b:Int)
		If style=GADGET_SLIDER	SetSliderRange button,a,b
	End Method
	
	Method SetValue:Int(val:Float)
		If style=GADGET_PROGBAR	
			UpdateProgBar button,val
			Local txt$=Text.Replace("<#value#>",Int(val*100))
			SetGadgetText label,txt
		End If

	End Method
	
	Method SetProp:Int(val:Int)
		If style=GADGET_SLIDER	
			SetSliderValue button,val		
			Local txt$=Text.Replace("<#value#>",SliderValue(button))
			SetGadgetText label,txt

		Return 0
		End If
	
		Local tw:Int=GadgetWidth(panel)
		Local ns:Int=tw*Float(val)/100
		SetGadgetShape label,0,0,ns,GadgetHeight(label)
		SetGadgetShape button,ns,0,tw-ns,GadgetHeight(label)

		Return 0
	End Method

	Function Create:tButtonExtra (msg$="",def:String="",x:Int,y:Int,w:Int,h:Int,group:tgadget,style:Int=GADGET_TEXTFIELD)
		Local sp:tButtonExtra =New tButtonExtra 
		Local txt$
		sp.panel=CreatePanel(x,y,w,h,group)
		sp.label=CreateLabel(msg,0,4,w/2,h-2,sp.panel)
		sp.text=msg
		SetGadgetText sp.label,msg

		If style=GADGET_TEXTFIELD
			sp.button=CreateTextField(w/2,0,w/2,h,sp.panel)
			SetGadgetText sp.button,def
		ElseIf style=GADGET_BUTTON
			sp.button=CreateButton(def,w/2,0,w/2,h,sp.panel)
		ElseIf style=GADGET_COMBOBOX
			sp.button=CreateComboBox(w/2,0,w/2,h,sp.panel)
		ElseIf style=GADGET_SLIDER
			sp.button=CreateSlider(w/2,0,w/2,h,sp.panel,SLIDER_HORIZONTAL|SLIDER_TRACKBAR)
			txt=sp.text.Replace("<#value#>",SliderValue(sp.button))
			SetGadgetText sp.label,txt
		ElseIf style=GADGET_PROGBAR
			sp.button=CreateProgBar(w/2,0,w/2,h-5,sp.panel)

			txt=sp.text.Replace("<#value#>","")
			SetGadgetText sp.label,txt

		End If

		SP.style=STYLE
		AddHook EmitEventHook,EventHandler,sp
		sp.SetProxy sp.panel
		
		
		
		SetGadgetLayout sp.panel,EDGE_ALIGNED,EDGE_ALIGNED,EDGE_ALIGNED,0
		SetGadgetLayout sp.button,EDGE_RELATIVE,EDGE_ALIGNED,EDGE_ALIGNED,0
		SetGadgetLayout sp.label,EDGE_ALIGNED,EDGE_RELATIVE,EDGE_ALIGNED,0

		Return sp
	End Function
	
	Method SETENABLED(sta:Int=True)
		If sta
			EnableGadget label
			EnableGadget button
			EnableGadget panel
		Else
			DisableGadget label
			DisableGadget button
			DisableGadget panel
		End If
	End Method
	
	Method SetColor(RED:Int,green:Int,blue:Int)
		SetGadgetColor label,RED,green,blue
		SetGadgetColor PANEL,RED,green,blue
		SetGadgetColor button,RED,green,blue

	End Method
	
	Method SetText:Int(tx:String)
		If tx="" Return 0
		
		'special cases
		
		If tx.toupper().contains("<#TFIELD#>")
			tx=tx.Replace("<#TFIELD#>","")
			SetGadgetText button,tx		
			Return 1
		End If
				
		If tx.toupper().contains("<#SIZE#>")
			tx=tx.Replace("<#SIZE#>","")
			
			Local tw:Int=GadgetWidth(panel)
			Local ns:Int=tw*Float(Int(tx))/100
			SetGadgetShape label,0,0,ns,GadgetHeight(label)
			SetGadgetShape button,ns,0,tw-ns,GadgetHeight(label)
			Return 1
		End If
		
		SetGadgetText label,tx
	End Method


	
	Function eventHandler:Object( pID%, pData:Object, pContext:Object )
		Local pEvent:TEvent =  TEvent(pData)
		Local obj:tButtonExtra = tButtonExtra (pContext)
		Local _data:Int,_extraData$
		
		If pEvent
		
			Select pevent.ID
				Case EVENT_GADGETACTION
					If PEVENT.SOURCE=OBJ.button
						
						If obj.style=GADGET_TEXTFIELD
							_EXTRAdata=GadgetText(obj.button)
						End If
						If obj.style=GADGET_SLIDER
							Local txt$
							txt=obj.text.Replace("<#value#>",SliderValue(obj.button))
							SetGadgetText obj.label,txt
							_data=SliderValue(obj.button)
						End If
						
						If obj.style=GADGET_COMBOBOX	_Data=SelectedGadgetItem(obj.button)
						

						EmitEvent CreateEvent(EVENT_GADGETACTION,obj,_data,0,0,0,_extradata)
						Return pevent
					End If
			End Select

			
		End If
		
		Return pData
	EndFunction


End Type

